#!/bin/bash
# ------------------------------------------------------------------
# This an automated tool to assist you in creating VLAN tagged 
# Client Access Interfaces in Oracle Database Appliance BareMetal
# For more information, please visit My Oracle Support 
# Note 1500709.1 & Note 2216567.1
# ------------------------------------------------------------------
#
#########################################################################
# Copyright:    2015 by Oracle, Inc. All Rights Reserved.
# Purpose:      Manages VLAN Tagged Interfaces in ODA BareMetal
# File:         ODA_vlan_config.sh
# Usage:        ODA_vlan_config.sh [add | del <vlan ID> ]
#               [ -i <interface> ] [ -ip <IPv4 addr> ] [ -n <netmask> ]
#               [ -g <gateway> ] 
#               OR [ help | man ]
#
# Authors:       Neeraj Gupta, Ruggero Citton
#########################################################################
# Bug fix: n/a
UPDATE_DATE='21-Dec-2016'
VERSION=1.03


###########################################################
# usage() procedure
###########################################################
# This is a procedure to display program usage information
# Called automatically when invalid arguments are passed
# User may call it by passing "help" as an argument
# > ODA_vlan_config.sh help

version() {
  $ECHO
  $ECHO "Oracle Database Appliance BareMetal VLAN Configurator"
  $ECHO "Copyright Oracle, Inc. 2015"
  $ECHO "version $VERSION Dated $UPDATE_DATE"
}

platform_check() {
  if [[ `/bin/uname -s` != 'Linux' ]]; then
    $ECHO "This tool only works on Linux nodes, exiting." 1>&2
    exit 1
  fi
}

oda_conf_check() {
  if [ -f /opt/oracle/oak/bin/oakcli ]; then
    if [[ `/opt/oracle/oak/bin/oakcli show env_hw` != "BM ODA"* ]]; then
      $ECHO "This tool works on ODA BareMetal only"
      $ECHO " for ODA Virtualized Platform use 'oakcli create vlan' command, exiting." 1>&2
      exit 1
    fi
  elif [ -f /opt/oracle/oak/bin/odaadmcli ]; then
    if [[ `/opt/oracle/oak/bin/odaadmcli show env_hw` != "BM ODA"* ]]; then
      $ECHO "This tool works on ODA BareMetal only"
      $ECHO " for ODA Virtualized Platform use 'oakcli create vlan' command, exiting." 1>&2
      exit 1
    fi
  else
    $ECHO "Looks like you are not on ODA environment, exiting" 1>&2
    exit 1
  fi
}

root_check() {
  $ECHO
  if [[ $EUID -ne 0 ]]; then
    $ECHO "You must be root to run this tool !" 1>&2
    exit 1
  fi
}

usage() {
  $ECHO "Usage: `basename $0` [ help | man | -h ] "
  $ECHO "       `basename $0` add "
  $ECHO "                            -v  <VLAN ID> "
  $ECHO "                            -i  <Interface> "
  $ECHO "                            -ip <IPv4 address> "
  $ECHO "                            -n  <Netmask> "
  $ECHO "                            -g  <Gateway> "
  $ECHO "                            -c "
  $ECHO "       `basename $0` del "
  $ECHO "                            -v  <VLAN ID> "
  $ECHO "                            -i  <Interface> "
  $ECHO "                           [-f] #only if the VLAN is not in use by the clusterware"
  $ECHO "         `basename $0` show "
  $ECHO ""
  $ECHO "Example: `basename $0` add -v 100 -i bond0 -ip 10.10.100.21 -n 255.255.255.0 -g 10.10.100.1"
  $ECHO "         `basename $0` del -v 100 -i bond0 "
  $ECHO "         `basename $0` del -v 100 -i bond0 -f"
  $ECHO ""
}

show_vlan() {
  $ECHO ""
  if [ `/sbin/lsmod | grep 8021q | wc -l` -gt 0 ]; then
    $ECHO "[Info] VLAN (8021q) support is enabled in system"
    cat /proc/net/vlan/config | grep -v VLAN > /tmp/_oda_vlan.$$
    if [ `cat /tmp/_oda_vlan.$$ | wc -l` -eq 0 ]; then
        $ECHO "[Info] No VLAN tagged interfaces found"
    else
        while read line
        do 
           IFACE=`$ECHO $line | awk '{print $1}'`
           $ECHO -n $line "| " 
           $ECHO -n `ifconfig $IFACE | grep -w inet | awk '{print $2}' | awk -F: '{print $NF}'`
           $ECHO -n " | " 
           $ECHO -n `ifconfig $IFACE | grep -w inet | awk -F: '{print $NF}'`
           $ECHO -n " | " 
           $ECHO `cat /sys/class/net/$IFACE/operstate`
        done < /tmp/_oda_vlan.$$
    fi
  else
    $ECHO "[Info] VLAN Module 8021q is not loaded"
    $ECHO "[Info] No VLAN Configurations found"
  fi
  $ECHO
}


add_vlan() {
  $ECHO ""
  # First process necessary CLI options
  COMM=0
  for cmd_operation in $*
  do
    case $1 in
        -v) VID=$2
            shift 2
            ;;
        -i) IFACE=$2
            shift 2
            ;;
        -ip) IPADDR=$2
            shift 2
            ;;
        -n) MASK=$2
            shift 2
            ;;
        -g) GW=$2
            shift 2
            ;;
        -c) COMM=1
            shift 1
            ;;
        -*) usage
            $ECHO "Invalid option: $1"
            exit
            ;;
        *)  if [ "$1" != "" ]; then
                usage
                $ECHO "Invalid argument: $1"
                exit
            fi
            ;;
     esac
  done

  # Validate CLI options
  if [ -z "$VID" ]; then
    usage
    $ECHO "Invalid argument: $1"
    $ECHO "Missing -v parameter (VLAN ID)"
    exit
  fi
  if [ $VID -lt 1 ] || [ $VID -gt 4095 ]; then
        $ECHO "Invalid VLAN ID. Must be between 1-4095. FAILED"
        exit 1
  fi

  if [ -z "$IFACE" ]; then
    usage
    $ECHO "Invalid argument: $1"
    $ECHO "Missing -i parameter (Interface)"
    exit
  fi
  if [ ! -d /sys/class/net/$IFACE ]; then
        $ECHO "Interface $IFACE does not exist. FAILED"
        exit 1
  fi

  if [ -z "$IPADDR" ]; then
    usage
    $ECHO "Invalid argument: $1"
    $ECHO "Missing -ip parameter (IPv4 address)"
    exit
  fi
  /bin/ipcalc -4 -c $IPADDR
  if [ $? -ne 0 ]; then
    $ECHO "Invalid IPv4 Address $IPADDR. FAILED"
    exit 1
  fi

  if [ -z "$MASK" ]; then
    usage
    $ECHO "Invalid argument: $1"
    $ECHO "Missing -n parameter (netmask)"
    exit
  fi
  /bin/ipcalc -4 -c $MASK
  if [ $? -ne 0 ]; then
    $ECHO "Invalid IPv4 Netmask $MASK. FAILED"
    exit 1
  fi

  if [ -z "$GW" ]; then
    usage
    $ECHO "Invalid argument: $1"
    $ECHO "Missing -g parameter (gateway)"
    exit
  fi
  /bin/ipcalc -4 -c $GW
  if [ $? -ne 0 ]; then
    $ECHO "Invalid IPv4 Gateway $GW. FAILED"
    exit 1
  fi
  NETWORK=`/bin/ipcalc -4 -n $IPADDR $MASK | awk -F= '{print $NF}'`
  PREFIX=`/bin/ipcalc -4 -p $IPADDR $MASK | awk -F= '{print $NF}'`
  GWNET=`/bin/ipcalc -4 -n $GW $MASK | awk -F= '{print $NF}'`

  # Check if the provided gateway actually matches subnet or not
  if [ "$NETWORK" != "$GWNET" ]; then
    $ECHO "Gateway $GW does not belong to subnet ${NETWORK}/${PREFIX}.  FAILED"
    exit 1
  fi

  # Calculate a unique table number
  LASTTABLE=`$ECHO $((\`/sbin/ip rule show | awk '{print $NF}'| egrep -v '(main|default|255|local)' | sort | tail -1\`))`
  if [ $LASTTABLE -lt 220 ]; then
    NEXTTABLE=221
  else
    NEXTTABLE=`expr $LASTTABLE + 1`
  fi
  $ECHO "Using iproute2 table ID: $NEXTTABLE"

  # Create IFCFG file now
  $ECHO "DEVICE=${IFACE}.${VID}" > /etc/sysconfig/network-scripts/ifcfg-${IFACE}.${VID}
  $ECHO "BOOTPROTO=static" >> /etc/sysconfig/network-scripts/ifcfg-${IFACE}.${VID}
  $ECHO "ONBOOT=yes" >> /etc/sysconfig/network-scripts/ifcfg-${IFACE}.${VID}
  $ECHO "IPADDR=$IPADDR" >> /etc/sysconfig/network-scripts/ifcfg-${IFACE}.${VID}
  $ECHO "NETMASK=$MASK" >> /etc/sysconfig/network-scripts/ifcfg-${IFACE}.${VID}
  $ECHO "GATEWAY=$GW" >> /etc/sysconfig/network-scripts/ifcfg-${IFACE}.${VID}
  $ECHO "VLAN=yes" >> /etc/sysconfig/network-scripts/ifcfg-${IFACE}.${VID}
  $ECHO "USERCTL=no" >> /etc/sysconfig/network-scripts/ifcfg-${IFACE}.${VID}
  $ECHO "IPV6INIT=no" >> /etc/sysconfig/network-scripts/ifcfg-${IFACE}.${VID}

  # Create the rule file now
  $ECHO "from ${IPADDR}/${PREFIX} table $NEXTTABLE" > /etc/sysconfig/network-scripts/rule-${IFACE}.${VID}
  $ECHO "to ${IPADDR}/${PREFIX} table $NEXTTABLE" >> /etc/sysconfig/network-scripts/rule-${IFACE}.${VID}

  # Create the route file now
  $ECHO "${NETWORK}/${PREFIX} dev ${IFACE}.${VID} table $NEXTTABLE" > /etc/sysconfig/network-scripts/route-${IFACE}.${VID}
  $ECHO "default via $GW dev ${IFACE}.${VID} table $NEXTTABLE" >> /etc/sysconfig/network-scripts/route-${IFACE}.${VID}

  $ECHO "Created config files in directory /etc/sysconfig/network-scripts"
  $ECHO "  ==> ifcfg-${IFACE}.${VID}\n  ==> rule-${IFACE}.${VID}\n  ==> route-${IFACE}.${VID}"

  $ECHO ""
  # Enable the interface now
  /sbin/ifup ${IFACE}.${VID}
  if [ $? -ne 0 ]; then
    $ECHO "Error occured while enabling $IFACE with VLAN $VID. FAILED"
    exit 1
  fi

  $ECHO
  $ECHO "Following VLAN(s) are now configured:"
  cat /proc/net/vlan/config | grep -v VLAN > /tmp/_oda_vlan.$$
  if [ `cat /tmp/_oda_vlan.$$ | wc -l` -eq 0 ]; then
      $ECHO "[Info] No VLAN tagged interfaces found"
  else
      while read line
      do 
          IFACE=`$ECHO $line | awk '{print $1}'`
          $ECHO -n $line "| " 
          $ECHO -n `ifconfig $IFACE | grep -w inet | awk '{print $2}' | awk -F: '{print $NF}'`
          $ECHO -n " | " 
          $ECHO -n `ifconfig $IFACE | grep -w inet | awk -F: '{print $NF}'`
          $ECHO -n " | " 
          $ECHO `cat /sys/class/net/$IFACE/operstate`
      done < /tmp/_oda_vlan.$$
  fi

}

del_vlan() {
  $ECHO ""
  # First process necessary CLI options
  DECOMM=0
  FORCE=0

  for cmd_operation in $*
  do
    case $1 in
    -v) VID=$2
        shift 2
        ;;
    -i) IFACE=$2
        shift 2
        ;;
    -f) FORCE=1
        shift 1
        ;;
    -*) usage
        $ECHO "Invalid option: $1"
        exit
        ;;
    *)  if [ "$1" != "" ]; then
        usage
        $ECHO "Invalid argument: $1"
        exit
        fi
        ;;
     esac
  done

  # Validate CLI options
  if [ -z "$VID" ]; then
    usage
    $ECHO "Invalid argument: $1"
    $ECHO "Missing -v parameter (VLAN ID)"
    exit
  fi
  if [ $VID -lt 1 ] || [ $VID -gt 4095 ]; then
    $ECHO "Invalid VLAN ID. Must be between 1-4095. FAILED"
    exit 1
  fi

  if [ -z "$IFACE" ]; then
    usage
    $ECHO "Invalid argument: $1"
    $ECHO "Missing -i parameter (Interface)"
    exit
  fi
  if [ ! -d /sys/class/net/$IFACE ]; then
    $ECHO "Interface $IFACE does not exist. FAILED"
    exit 1
  fi

  if [ ! -d /sys/class/net/${IFACE}.${VID} ]; then
    $ECHO "Interface $IFACE with VLAN $VID is not present. FAILED"
    exit 1
  fi

  if [ "$FORCE" -ne 1 ]; then
    if [ "$SRVCTL" = "" ]; then
      $ECHO "[Warning] SRVCTL not defined as an environment variable" 
      $ECHO "[Info] VLAN Delete function can not be run without '-f' option"
      $ECHO
      exit 1
    fi
  
    if [ ! -f $SRVCTL ]; then
      $ECHO "[Error] SRVCTL=$SRVCTL is an invalid environment variable"
      $ECHO
      exit 1
    fi
  
    $SRVCTL config network > /dev/null
    if [ $? -ne 0 ]; then
      $ECHO "[Error] Could not determine clusterware network config"
      $ECHO "[Error] Please check $SRVCTL usage manually"
      $ECHO
      exit 1
    fi  

    if [ `$SRVCTL config network | grep ${IFACE}.${VID} | wc -l` -ne 0 ]; then
      $ECHO "${IFACE}.${VID} is already configured in Oracle Database Appliance"
      $ECHO "  To remove it, first unconfigure from Clusterware"
      exit 1
    fi
  fi

  # Now remove OS level VLAN setup
  IFCFG=/etc/sysconfig/network-scripts/ifcfg-${IFACE}.${VID}
  RULE=/etc/sysconfig/network-scripts/rule-${IFACE}.${VID}
  ROUTE=/etc/sysconfig/network-scripts/route-${IFACE}.${VID}
  mkdir -p /etc/sysconfig/network-scripts/bak

  /sbin/ifdown ${IFACE}.${VID}
  if [ $? -ne 0 ]; then
    $ECHO "Failed to bring down $IFACE with VLAN $VID"
    $ECHO "Please check for $IFCFG"
    exit 1
  fi

  if [ -f $IFCFG ]; then
    mv $IFCFG `dirname $IFCFG`/bak
    $ECHO "Removed $IFCFG " 
  fi

  if [ -f $RULE ]; then
    mv $RULE `dirname $RULE`/bak
    $ECHO "Removed $RULE "
  fi

  if [ -f $ROUTE ]; then
        mv $ROUTE `dirname $ROUTE`/bak
    $ECHO "Removed $ROUTE "
  fi
}


main() {
  for cmd_operation in $*
  do
    case $1 in
            man)  sed -n 's/^#M#//p' $0  | more
                  exit
                  ;;
            help) usage
                  exit
                  ;;
            show) show_vlan
                  exit
                  ;;
            add)  shift 1
                  add_vlan $*
                  exit
                  ;;
            del)  shift 1
                  del_vlan $*
                  exit
                  ;;
            -h)   usage
                  exit
                  ;;
            -*)   usage
                  $ECHO "Invalid option: $1"
                  exit
                  ;;
             *)   if [ "$1" != "" ]; then
                    usage
                    $ECHO "Invalid argument: $1"
                    exit
                 fi
                 ;;
    esac
  done
}


# MAIN
SRVCTL=`env | grep SRVCTL | awk -F= '{print $2}'`

if [ `uname -s` = "Linux" ]; then
      ECHO="echo -e "
else
      ECHO="echo "
fi

version
root_check
platform_check
oda_conf_check
main $*

# ------------------------------------------------------------------
# End Of File
# ------------------------------------------------------------------

